#ifndef _H_CAxisAngle
#define _H_CAxisAngle
#pragma once

// the identity rotation is defined as { (1.0, 0.0, 0.0), 0.0 }

const float		kAxisAngle_DefaultXCoord					= 1.0f;
const float		kAxisAngle_DefaultYCoord					= 0.0f;
const float		kAxisAngle_DefaultZCoord					= 0.0f;
const float		kAxisAngle_DefaultAngle						= 0.0f;

// This is our 3D point class + angle.
class CAxisAngle 
{
public:
	// default constructor
	inline				CAxisAngle() :
							x(kAxisAngle_DefaultXCoord),
							y(kAxisAngle_DefaultYCoord),
							z(kAxisAngle_DefaultZCoord),
							angle(kAxisAngle_DefaultAngle)
						{
						}
						
						CAxisAngle(
							float	inX,
							float	inY,
							float	inZ,
							float	inAngle) :
								x(inX),
								y(inY),
								z(inZ),
								angle(inAngle)
						{
						}

	void				clear_rotation()
						{
							x		= kAxisAngle_DefaultXCoord;
							y		= kAxisAngle_DefaultYCoord;
							z		= kAxisAngle_DefaultZCoord;
							angle	= kAxisAngle_DefaultAngle;
						}
	
	inline bool			is_rotated() const
						{
							return (
								x != kAxisAngle_DefaultXCoord
								|| y != kAxisAngle_DefaultYCoord
								|| z != kAxisAngle_DefaultZCoord
								|| angle != kAxisAngle_DefaultAngle
							);
						}
	
	// rotates this axis angle by another axis angle
	inline void			rotate_by(
							const CAxisAngle&	inRotateBy)
								{ rotate_axis_angle_by(inRotateBy, *this); }
	
	void				rotate_by(
							float				inX,
							float				inY,
							float				inZ);
	
	inline bool			operator==(const CAxisAngle& inRhs)
							{ return x == inRhs.x && y == inRhs.y && z == inRhs.z && angle == inRhs.angle; }

	inline bool			operator!=(const CAxisAngle& inRhs)
							{ return x != inRhs.x || y != inRhs.y || z != inRhs.z || angle != inRhs.angle; }

// STATIC FUNCTIONS

	// converts the specified axis angle to an array of 
	// four floats representing a quaternion
	static void			axis_angle_to_quat(
							const CAxisAngle&	inAxisAngle,
							float*				outQ);
	
	// rotates ioAxisAngle by the axis angle inRotateBy
	static void			rotate_axis_angle_by(
							const CAxisAngle&	inRotateBy,
							CAxisAngle&			ioAxisAngle);
	

// MEMBER VARIABLES
public:
	float x, y, z, angle;
};

#endif
