// ===========================================================================
//	CFloatRect					(c) 2013 Mark F. Coniglio. All rights reserved
// ===========================================================================

#ifndef _H_CFloatRect
#define _H_CFloatRect
#pragma once

#define PTRECT_FLTMIN(a,b) ((a) < (b)) ? (a) : (b)
#define PTRECT_FLTMAX(a,b) ((a) > (b)) ? (a) : (b)

#include "CFloatXY.h"

class CFloatRect
{
public:
								CFloatRect() :
									left(+FLT_MAX),
									top(+FLT_MAX),
									right(-FLT_MAX),
									bottom(-FLT_MAX)
								{
								}
							
								CFloatRect(
									float	inLeft,
									float	inTop,
									float	inRight,
									float	inBottom) :
										left(inLeft),
										top(inTop),
										right(inRight),
										bottom(inBottom)
								{
								}
	
								CFloatRect(
									const Rect&			inQDRect) :
										left(inQDRect.left),
										top(inQDRect.top),
										right(inQDRect.right),
										bottom(inQDRect.bottom)
								{
								}
	
	CFloatRect&					operator=(
									const Rect&			inQDRect)
								{
									left = inQDRect.left;
									top = inQDRect.top;
									right = inQDRect.right;
									bottom = inQDRect.bottom;
									return *this;
								}
	
								operator Rect() const
								{
									Rect r;
									r.left = left;
									r.top = top;
									r.right = right;
									r.bottom = bottom;
									return r;
								}
	
	bool						operator==(
									const CFloatRect&	inRhs) const
								{
									return left == inRhs.left && right == inRhs.right && top == inRhs.top && bottom == inRhs.bottom;
								}
				
	bool						operator!=(
									const CFloatRect&	inRhs) const
								{
									return left != inRhs.left || right != inRhs.right || top != inRhs.top || bottom != inRhs.bottom;
								}
				
	inline CFloatRect			operator+(
									const CFloatRect&	inRhs) const
								{
									return CFloatRect(
												left   + inRhs.left,
												top    + inRhs.top,
												right  + inRhs.right,
												bottom + inRhs.bottom);
								}
	
	inline CFloatRect			operator-(
									const CFloatRect&	inRhs) const
								{
									return CFloatRect(
												left   - inRhs.left,
												top    - inRhs.top,
												right  - inRhs.right,
												bottom - inRhs.bottom);
								}
	
	inline CFloatRect&			operator+=(
									const CFloatRect&	inRhs)
								{
									left   += inRhs.left;
									top    += inRhs.top;
									right  += inRhs.right;
									bottom += inRhs.bottom;
									
									return (*this);
								}
	
	inline CFloatRect&			operator-=(
									const CFloatRect&	inRhs)
								{
									left   -= inRhs.left;
									top    -= inRhs.top;
									right  -= inRhs.right;
									bottom -= inRhs.bottom;

									return (*this);
								}

// addition/subtraction operator with CFloatXY = offset

	inline CFloatRect			operator+(
									const CFloatXY&		inRhs) const
								{
									return CFloatRect(
												left   + inRhs.x,
												top    + inRhs.y,
												right  + inRhs.x,
												bottom + inRhs.y);
								}
	
	inline CFloatRect			operator-(
									const CFloatXY&		inRhs) const
								{
									return CFloatRect(
												left   - inRhs.x,
												top    - inRhs.y,
												right  - inRhs.x,
												bottom - inRhs.y);
								}
	
	inline CFloatRect&			operator+=(
									const CFloatXY&		inRhs)
								{
									left   += inRhs.x;
									top    += inRhs.y;
									right  += inRhs.x;
									bottom += inRhs.y;
									
									return (*this);
								}
	
	inline CFloatRect&			operator-=(
									const CFloatXY&		inRhs)
								{
									left   -= inRhs.x;
									top    -= inRhs.y;
									right  -= inRhs.x;
									bottom -= inRhs.y;

									return (*this);
								}
	
// addition/subtraction operator with float = offset

	inline CFloatRect			operator+(
									const float			inOffset) const
								{
									return CFloatRect(
												left   + inOffset,
												top    + inOffset,
												right  + inOffset,
												bottom + inOffset);
								}
	
	inline CFloatRect			operator-(
									const float			inOffset) const
								{
									return CFloatRect(
												left   - inOffset,
												top    - inOffset,
												right  - inOffset,
												bottom - inOffset);
								}
	
	inline CFloatRect&			operator+=(
									const float			inOffset)
								{
									left   += inOffset;
									top    += inOffset;
									right  += inOffset;
									bottom += inOffset;
									
									return (*this);
								}
	
	inline CFloatRect&			operator-=(
									const float			inOffset)
								{
                                    left   -= inOffset;
                                    top    -= inOffset;
                                    right  -= inOffset;
                                    bottom -= inOffset;

									return (*this);
								}

// multiplication operator with CFloatXY = scale

	inline CFloatRect			operator*(
									const CFloatXY&			inScale)
								{
									CFloatRect result = *this;
									
									result *= inScale;
									
									return result;
								}
								
	inline CFloatRect&			operator*=(
									const CFloatXY&			inScale)
								{
									const CFloatXY center = GetCenter();
									
									// translate center to the origin
									(*this) -= center;
									
									// scale
									left	*= inScale.x;
									top		*= inScale.y;
									right	*= inScale.x;
									bottom	*= inScale.y;
									
									// translate center back to orginal position
									(*this) += center;
									
									return (*this);
								}
								
// multiplication operator with float = scale
	inline CFloatRect			operator*(
									const float				inScale)
								{
									CFloatRect result = *this;
									result *= inScale;
									return result;
								}
								
	inline CFloatRect&			operator*=(
									const float				inScale)
								{
									const CFloatXY center = GetCenter();
									
									// translate center to the origin
									(*this) -= center;
									
									// scale
									left	*= inScale;
									right	*= inScale;
									top		*= inScale;
									bottom	*= inScale;
									
									// translate center back to orginal position
									(*this) += center;

									return *this;
								}


	// CORNER POINT GETTERS
	inline CFloatXY				GetTopLeft() const
									{ return CFloatXY(left, top); }
	
	inline CFloatXY				GetTopRight() const
									{ return CFloatXY(right, top); }
	
	inline CFloatXY				GetBottomLeft() const
									{ return CFloatXY(left, bottom); }
	
	inline CFloatXY				GetBottomRight() const
									{ return CFloatXY(right, bottom); }
	

	// CORNER POINT SETTERS
	inline void					SetTopLeft(
									CFloatXY	inTopLeft)
										{ left = inTopLeft.x; top = inTopLeft.y; }
	
	inline void					SetTopRight(
									CFloatXY	inTopRight)
										{ right = inTopRight.x; top = inTopRight.y; }
	
	inline void					SetBottomLeft(
									CFloatXY	inBottomLeft)
										{ left = inBottomLeft.x; bottom = inBottomLeft.y; }
	
	inline void					SetBottomRight(
									CFloatXY	inBottomRight)
										{ right = inBottomRight.x; bottom = inBottomRight.y; }
	
	// CENTER + AREA

	inline CFloatXY				GetCenter() const
									{ return CFloatXY(left+(right-left)/2.0f, top+(bottom-top)/2.0f); }
									
	inline float				GetArea() const
									{ return width() * height(); }
									
	inline CFloatRect&			MoveCenterTo(
									const CFloatXY&			inNewCenter)
								{
									return (*this) += (inNewCenter - GetCenter());
								}
	
	// WIDTH + HEIGHT
	inline float				width() const
									{ return right - left; }
									
	inline void					SetWidth(
									float					inWidth)
										{ right = left + inWidth; }
								
	inline float				height() const
									{ return bottom - top; }
									
	inline void					SetHeight(
									float					inHeight)
										{ bottom = top + inHeight; }
	
	// SIZE
	
	inline CFloatXY				GetSize() const
									{ return CFloatXY(right-left, bottom-top); }
	
	inline CFloatRect&			SetSize(
									const CFloatXY&			inNewSize)
								{
									return (*this) *= (inNewSize / GetSize());
								}
	
	// RESET
	inline void					SetZero()
								{
									left	= 0.0f;
									top		= 0.0f;
									right	= 0.0f;
									bottom	= 0.0f;
								}
	
	inline void					Clear()
								{
									left	= +FLT_MAX;
									top		= +FLT_MAX;
									right	= -FLT_MAX;
									bottom	= -FLT_MAX;
								}
	
	// CHARACTERISTICS
	inline bool					IsUnitRect() const
								{
									return (left == 0.0f && top == 0.0f && right == 1.0f && bottom == 1.0f);
								}
	
	inline bool					IsEmpty() const
								{
									return (left >= right || top >= bottom);
								}

	// POINT TESTS
	inline bool					ContainsPoint(
									const CFloatXY&			inPoint) const
								{
									return (inPoint.x >= left && inPoint.x <= right && inPoint.y >= top && inPoint.y <= bottom);
								}
	
	inline bool					ConstrainPoint(
									CFloatXY&			inPoint) const
								{
									bool didConstrain = false;
									if (inPoint.x < left) {
										inPoint.x = left;
										didConstrain = true;
									} else if (inPoint.x > right) {
										inPoint.x = right;
										didConstrain = true;
									}
									if (inPoint.y < top) {
										inPoint.y = top;
										didConstrain = true;
									} else if (inPoint.y > bottom) {
										inPoint.y = bottom;
										didConstrain = true;
									}
									
									return didConstrain;
								}

	// GEOMETRY OPERATIONS
	inline CFloatRect&			Inset(
									const CFloatXY&			inPoint)
								{
									left	+= inPoint.x;
									right	-= inPoint.x;
									top		+= inPoint.y;
									bottom	-= inPoint.y;
									return (*this);
								}
						
	inline CFloatRect&			Inset(
									const float				inX,
									const float				inY)
								{
									left	+= inX;
									right	-= inX;
									top		+= inY;
									bottom	-= inY;
									return (*this);
								}
						
	inline CFloatRect&			Inset(
									const float				inAmount)
								{
									left	+= inAmount;
									right	-= inAmount;
									top		+= inAmount;
									bottom	-= inAmount;
									return (*this);
								}
						
	inline CFloatRect&			Expand(
									const CFloatXY&			inPoint)
								{
									left	-= inPoint.x;
									right	+= inPoint.x;
									top		-= inPoint.y;
									bottom	+= inPoint.y;
									return (*this);
								}
						
	inline CFloatRect&			Expand(
									const float				inX,
									const float				inY)
								{
									left	-= inX;
									right	+= inX;
									top		-= inY;
									bottom	+= inY;
									return (*this);
								}
						
	inline void					ExpandBoundsToIncludePoint(
									const CFloatXY&			inPoint)
								{
									if (inPoint.x < left)
										left = inPoint.x;
									if (inPoint.x > right)
										right = inPoint.x;
									if (inPoint.y < top)
										top = inPoint.y;
									if (inPoint.y > bottom)
										bottom = inPoint.y;
								}
						
	inline CFloatRect&			Union(
									const CFloatRect&  inNewRect)
								{
									const bool curIsEmpty = this->IsEmpty();
									const bool newIsEmpty = inNewRect.IsEmpty();
									
									if (curIsEmpty == false && newIsEmpty == false) {
										left	= PTRECT_FLTMIN(left, inNewRect.left);
										top		= PTRECT_FLTMIN(top, inNewRect.top);
										right	= PTRECT_FLTMAX(right, inNewRect.right);
										bottom	= PTRECT_FLTMAX(bottom, inNewRect.bottom);
									} else if (curIsEmpty == true && newIsEmpty == false) {
										*this = inNewRect;
									} else if (curIsEmpty == false && newIsEmpty == true) {
										// new rect is empty and we are not, so we 
										// don't need to do anything
									} else {
										// if both rects are empty
										left = top = right = bottom = 0;
									}
									
									return (*this);
								}

								#define __fr__min2(a,b) ((a) < (b)) ? (a) : (b)
								#define __fr__max2(a,b) ((a) > (b)) ? (a) : (b)

private:
	static inline void			_sect_lines_(
									float	a1,
									float	a2,
									float	b1,
									float	b2,
									float&	o1,
									float&	o2)
								{
									o1 = o2 = 0;
									// if the two line lengths are > 0
									if ((a1) < (a2) && (b1) < (b2)) {
										// if startpoint of segment a is to the left of b, and endpoint of a > startpoint of b
										if ((a1) <= (b1) && (a2) > (b1)) {
											o1 = __fr__max2(a1, b1);
											o2 = __fr__min2(a2, b2);
										// if startpoint of segment b is to the left of a, and endpoint of a > startpoint of b
										} else if ((b1) <= (a1) && (b2) > (a1)) {
											o1 = __fr__max2(a1, b1);
											o2 = __fr__min2(a2, b2);
										}
									}
								}

public:
	inline bool					IsIntersection(
									const CFloatRect&	s2)
								{
									CFloatRect d;
									_sect_lines_(left, right, s2.left, s2.right, d.left, d.right);
									_sect_lines_(top, bottom, s2.top, s2.bottom, d.top, d.bottom);
									if (d.left >= d.right || d.top >= d.bottom) { d.left = d.right = d.top = d.bottom = 0; }
									return !d.IsEmpty();
								}
	
	inline bool					GetIntersection(
									const CFloatRect&	s2,
									CFloatRect*			d)
								{
									_sect_lines_(left, right, s2.left, s2.right, d->left, d->right);
									_sect_lines_(top, bottom, s2.top, s2.bottom, d->top, d->bottom);
									if (d->left >= d->right || d->top >= d->bottom) { d->left = d->right = d->top = d->bottom = 0; }
									return !d->IsEmpty();
								}
	
	static bool					IsIntersection(
									const CFloatRect&	s1,
									const CFloatRect&	s2)
								{
									CFloatRect d;
									_sect_lines_(s1.left, s1.right, s2.left, s2.right, d.left, d.right);
									_sect_lines_(s1.top, s1.bottom, s2.top, s2.bottom, d.top, d.bottom);
									if (d.left >= d.right || d.top >= d.bottom) { d.left = d.right = d.top = d.bottom = 0; }
									return !d.IsEmpty();
								}
	
	static inline bool			GetIntersection(
									const CFloatRect&	s1,
									const CFloatRect&	s2,
									CFloatRect*			d)
								{
									_sect_lines_(s1.left, s1.right, s2.left, s2.right, d->left, d->right);
									_sect_lines_(s1.top, s1.bottom, s2.top, s2.bottom, d->top, d->bottom);
									if (d->left >= d->right || d->top >= d->bottom) { d->left = d->right = d->top = d->bottom = 0; }
									return !d->IsEmpty();
								}
	
	static inline void			Union(
									const CFloatRect&  s1,
									const CFloatRect&  s2,
									CFloatRect *       d)
								{
									const bool empty1 = s1.IsEmpty();
									const bool empty2 = s2.IsEmpty();
									
									if (empty1 == false && empty2 == false) {
										(d)->left	= PTRECT_FLTMIN(s1.left, s2.left);
										(d)->top	= PTRECT_FLTMIN(s1.top, s2.top);
										(d)->right	= PTRECT_FLTMAX(s1.right, s2.right);
										(d)->bottom	= PTRECT_FLTMAX(s1.bottom, s2.bottom);
									} else if (empty1 == true && empty2 == false) {
										*d = s2;
									} else if (empty1 == false && empty2 == true) {
										*d = s1;
									} else {
										(d)->left = (d)->top = (d)->right = (d)->bottom = 0;
									}
								}

	static inline CFloatRect	Union(
									const CFloatRect&  s1,
									const CFloatRect&  s2)
								{
									CFloatRect outRect;
									
									const bool empty1 = s1.IsEmpty();
									const bool empty2 = s2.IsEmpty();
									
									if (empty1 == false && empty2 == false) {
										outRect.left	= PTRECT_FLTMIN(s1.left, s2.left);
										outRect.top		= PTRECT_FLTMIN(s1.top, s2.top);
										outRect.right	= PTRECT_FLTMAX(s1.right, s2.right);
										outRect.bottom	= PTRECT_FLTMAX(s1.bottom, s2.bottom);
									} else if (empty1 == true && empty2 == false) {
										outRect = s2;
									} else if (empty1 == false && empty2 == true) {
										outRect = s1;
									} else {
										outRect.left = outRect.top = outRect.right = outRect.bottom = 0;
									}
									
									return outRect;
								}

	inline void		ScaleDestRectToSourceAspectRatio(
						double			inSrcWidth,
						double			inSrcHeight,
						double*			outScaleFactor = NULL)
					{
						// scale = min( dst.width/src.width, dst.height/src.height)
						
						const double origDstWidth = width();
						const double origDstHeight = height();
						
						const double widthDstOverSrc = origDstWidth/inSrcWidth;
						const double heightDstOverSrc = origDstHeight/inSrcHeight;
						
						double scale = (widthDstOverSrc < heightDstOverSrc) ? widthDstOverSrc : heightDstOverSrc;
						if (outScaleFactor != NULL) {
							*outScaleFactor = scale;
						}
						
						double newDstWidth = inSrcWidth * scale;
						double newDstHeight = inSrcHeight * scale;
						
						double horzInset = (origDstWidth - newDstWidth) / 2.0;
						double vertInset = (origDstHeight - newDstHeight) / 2.0;
						
						left += horzInset;
						right -= horzInset;
						top += vertInset;
						bottom -= vertInset;
					}
	
	float	left;
	float	top;
	float	right;
	float	bottom;
	
};
				

#endif
