// ===========================================================================
//	CFloatXYZ					(C)2009 Mark F. Coniglio. All rights reserved.
// ===========================================================================

#ifndef _H_CFLOATXYZ
#define _H_CFLOATXYZ

#include "CPlusPlusMacros.h"
#include "izzy_math.h"

// ---------------------------------------------------------------------------
//		CFloatXYZ
// ---------------------------------------------------------------------------
CLASSHDR CFloatXYZ
{
PUBLIC

#ifdef __cplusplus

	inline				CFloatXYZ()
						{
							x = y = z = 0.0f;
						}
	
	inline				CFloatXYZ(
							const CFloatXYZ&		inXYZ)
						{
							x = inXYZ.x;
							y = inXYZ.y;
							z = inXYZ.z;
						}
	
	inline				CFloatXYZ(
							const float*			inXYZArray)
						{
							x = inXYZArray[0];
							y = inXYZArray[1];
							z = inXYZArray[2];
						}
				
	inline				CFloatXYZ(
							float	inX,
							float	inY,
							float	inZ)
						{
							x = inX;
							y = inY;
							z = inZ;
						}
	
	inline void			set_origin()
						{
							x = y = z = 0.0f;
						}
						
	inline bool			is_at_origin()
						{
							return (x == 0.0f && y == 0.0f && z == 0.0f);
						}
						
	inline void			set(
							float	inX,
							float	inY,
							float	inZ)
						{
							x = inX;
							y = inY;
							z = inZ;
						}
	
	inline CFloatXYZ&	limit(
							float	inMin,
							float	inMax)
						{
							if (x < inMin) x = inMin;
							else if (x > inMax) x = inMax;
							
							if (y < inMin) y = inMin;
							else if (y > inMax) y = inMax;
							
							if (z < inMin) z = inMin;
							else if (z > inMax) z = inMax;
							
							return *this;
						}
						
	inline CFloatXYZ&	operator =(
							const CFloatXYZ&	inValue)
						{
							x = inValue.x;
							y = inValue.y;
							z = inValue.z;
							return *this;
						}
						
	inline CFloatXYZ&	operator =(
							const float			inValue)
						{
							x = y = z = inValue;
							return *this;
						}
						
	// --------------------
	// float*
	// --------------------
						operator const float*() const
								{ return &x; }
	// --------------------
	// ADDITION
	// --------------------
	
	inline CFloatXYZ	operator +(
							float inValue) const
								{ return CFloatXYZ(x + inValue, y + inValue, z + inValue); }
				
	
	inline CFloatXYZ	operator +(
							const CFloatXYZ&	inValue) const
								{ return CFloatXYZ(x + inValue.x, y + inValue.y, z + inValue.z); }
				
	
	inline CFloatXYZ&	operator +=(
							float inValue)
						{
							x += inValue;
							y += inValue;
							z += inValue;
							return (*this);
						}
						
	
	inline CFloatXYZ&	operator +=(
							const CFloatXYZ&	inValue)
						{
							x += inValue.x;
							y += inValue.y;
							z += inValue.z;
							return (*this);
						}
				
	// --------------------
	// SUBTRACTION
	// --------------------
	
	inline CFloatXYZ	operator -() const
							{ return CFloatXYZ(-x, -y, -z); }
							
	inline CFloatXYZ	operator -(
							float inValue) const
								{ return CFloatXYZ(x - inValue, y - inValue, z - inValue); }
						
	
	inline CFloatXYZ	operator -(
							const CFloatXYZ&	inValue) const
								{ return CFloatXYZ(x - inValue.x, y - inValue.y, z - inValue.z); }
						
	
	inline CFloatXYZ&	operator -=(
							float inValue)
						{
							x -= inValue;
							y -= inValue;
							z -= inValue;
							return (*this);
						}
				
	
	inline CFloatXYZ&	operator -=(
							const CFloatXYZ&	inValue)
						{
							x -= inValue.x;
							y -= inValue.y;
							z -= inValue.z;
							return (*this);
						}
				
	
	// --------------------
	// MULTIPLICATION
	// --------------------
	
	inline CFloatXYZ	operator *(
							float inValue) const
								{ return CFloatXYZ(x * inValue, y * inValue, z * inValue); }
				
	inline CFloatXYZ	operator *(
							const CFloatXYZ&	inValue) const
								{ return CFloatXYZ(x * inValue.x, y * inValue.y, z * inValue.z); }
				
	inline CFloatXYZ&	operator *=(
							float inValue)
						{
							x *= inValue;
							y *= inValue;
							z *= inValue;
							return (*this);
						}
				
	inline CFloatXYZ&	operator *=(
							const CFloatXYZ&	inValue)
						{
							x *= inValue.x;
							y *= inValue.y;
							z *= inValue.z;
							return (*this);
						}
				
	// --------------------
	// DIVISION
	// --------------------
	
	inline CFloatXYZ	operator /(
							float inValue) const
								{ return CFloatXYZ(x / inValue, y / inValue, z / inValue); }
				
	inline CFloatXYZ	operator /(
							const CFloatXYZ&	inValue) const
								{ return CFloatXYZ(x / inValue.x, y / inValue.y, z / inValue.z); }
						
	inline CFloatXYZ&	operator /=(
							float inValue)
						{
							x /= inValue;
							y /= inValue;
							z /= inValue;
							return (*this);
						}
				
	inline CFloatXYZ&	operator /=(
							const CFloatXYZ&	inValue)
						{
							x /= inValue.x;
							y /= inValue.y;
							z /= inValue.z;
							return (*this);
						}
	
	// --------------------
	// COMPARISONS
	// --------------------
	
	inline bool			operator ==(
							const CFloatXYZ&	inValue) const
								{ return x == inValue.x && y == inValue.y && z == inValue.z; }
	
	inline bool			operator !=(
							const CFloatXYZ&	inValue) const
								{ return x != inValue.x || y != inValue.y || z != inValue.z; }
	// --------------------
	// GEOMETRIC OPERATIONS
	// --------------------
	
	inline float		magnitude() const
							{ return std::sqrt((x * x) + (y * y) + (z * z)); }
						
	inline float		distance(const CFloatXYZ& inRhs) const
						{
							CFloatXYZ p = (*this) - inRhs;
							return p.magnitude();
						}
	
	inline CFloatXYZ&	normalize()
						{
							float d = magnitude();
							if (d != 0.0f) {
								x /= d;
								y /= d;
								z /= d;
							} else {
								x = y = z = INFINITY;
							}
							return *this;
						}
	
	inline CFloatXYZ	interpolate(
							const CFloatXYZ&	inRhs,
							const float			t) const
						{
							return CFloatXYZ(x + (inRhs.x - x) * t, y + (inRhs.y - y) * t, z + (inRhs.z - z) * t);
						}

	inline void			rotate_about_x_degrees(
							const float	inAngleDegrees)
						{
							rotate_about_x_radians(float(DEG2RAD(inAngleDegrees)));
						}

	inline void			rotate_about_x_radians(
							const float	inAngleRadians)
						{
							const float sinTheta = sin(inAngleRadians);
							const float cosTheta = cos(inAngleRadians);
							
							// rotate about the x-axis
							// x' = x
							// y' = y*cos q - z*sin q
							// z' = y*sin q + z*cos q
							
							const float y1 = y * cosTheta - z * sinTheta; 
							const float z1 = y * sinTheta + z * cosTheta;
							
							y = y1;
							z = z1;
						}
						
	inline void			rotate_about_y_degrees(
							const float	inAngleDegrees)
						{
							rotate_about_y_radians(float(DEG2RAD(inAngleDegrees)));
						}

	inline void			rotate_about_y_radians(
							const float	inAngleRadians)
						{
							const float sinTheta = sin(inAngleRadians);
							const float cosTheta = cos(inAngleRadians);
							
							// rotate about the y-axis
							// x' = z*sin q + x*cos q
							// y' = y
							// z' = z*cos q - x*sin q
							
							const float x1 = z * sinTheta + x * cosTheta;
							const float z1 = z * cosTheta - x * sinTheta;
							
							x = x1;
							z = z1;
						}
						
	inline void			rotate_about_z_degrees(
							const float	inAngleDegrees)
						{
							rotate_about_z_radians(float(DEG2RAD(inAngleDegrees)));
						}

	inline void			rotate_about_z_radians(
							const float	inAngleRadians)
						{
							const float sinTheta = sin(inAngleRadians);
							const float cosTheta = cos(inAngleRadians);
							
							// rotate about the z-axis
							// x' = x*cos q - y*sin q
							// y' = x*sin q + y*cos q 
							// z' = z
							
							const float x1 = x * cosTheta - y * sinTheta; 
							const float y1 = x * sinTheta + y * cosTheta;
							
							x = x1;
							y = y1;
						}
						
	inline CFloatXYZ crossProduct(
						const CFloatXYZ&	b) const
					{
						CFloatXYZ p;
						p.x = (y * b.z) - (z * b.y);
						p.y = (z * b.x) - (x * b.z);
						p.z = (x * b.y) - (y * b.x);
						return p;
					}
	
	static
	inline CFloatXYZ computeNormal(
						const CFloatXYZ&		p0,
						const CFloatXYZ&		p1,
						const CFloatXYZ&		p2)
					{
						CFloatXYZ U = p1 - p0;
						CFloatXYZ V = p2 - p0;
						CFloatXYZ normal;
						normal.x = (U.y * V.z) - (U.z * V.y);
						normal.y = (U.z * V.x) - (U.x * V.z);
						normal.z = (U.x * V.y) - (U.y * V.x);
						
						normal.normalize();
						
						return normal;
					}
	
	// --------------------
	// ENDIAN SWAPPING
	// --------------------
	
	CFloatXYZ			get_native_from_little() const
						{
							#if TARGET_RT_BIG_ENDIAN
								return CFloatXYZ(endian_swap_float(x), endian_swap_float(y), endian_swap_float(z));
							#else
								return *this;
							#endif
						}
						
	CFloatXYZ			get_little_from_native() const
						{
							#if TARGET_RT_BIG_ENDIAN
								return CFloatXYZ(endian_swap_float(x), endian_swap_float(y), endian_swap_float(z));
							#else
								return *this;
							#endif
						}
						
	CFloatXYZ			get_big_from_native() const
						{
							#if TARGET_RT_LITTLE_ENDIAN
								return CFloatXYZ(endian_swap_float(x), endian_swap_float(y), endian_swap_float(z));
							#else
								return *this;
							#endif
						}
						
	CFloatXYZ			get_native_from_big() const
						{
							#if TARGET_RT_LITTLE_ENDIAN
								return CFloatXYZ(endian_swap_float(x), endian_swap_float(y), endian_swap_float(z));
							#else
								return *this;
							#endif
						}
						
	// --------------------
	// DEBUGGING
	// --------------------
	inline bool			has_nan_value() const
						{
							return is_nan(x) || is_nan(y) || is_nan(z);
						}	
	
#endif	// #ifdef __cplusplus

	float	x;
	float	y;
	float	z;
};



#endif
