// ===========================================================================
//	CMutex						(c) 2006 Mark F. Coniglio. All rights reserved
// ===========================================================================

#ifndef _H_CMutex
#define _H_CMutex
#pragma once

#include "IZTime.h"

#if TARGET_OS_MAC
#include <pthread.h>
#endif

#if TARGET_OS_WIN
#define	bzero(ptr, len)	memset((ptr), 0, (len))
#endif

#ifdef __ISADORA_APP__
	#define MUTEX_KEEP_MAIN_THREAD_LOCK_COUNT	TRUE
	#define	MUTEX_BACKTRACE_ENABLE				(FALSE && DEBUG)
#else
	#define MUTEX_KEEP_MAIN_THREAD_LOCK_COUNT	FALSE
	#define	MUTEX_BACKTRACE_ENABLE				FALSE
#endif

class CMutex
{
public:
			CMutex(
				bool	inRecursive = false);
	
			~CMutex();
	
	SInt32	Lock();			// returns lock count after lock

	SInt32	Unlock();		// returns lock count after unlock
	
	SInt32	GetLockCount() const
				{ return mLockCount; }

	#if MUTEX_KEEP_MAIN_THREAD_LOCK_COUNT
	SInt32	GetMainThreadLockCount() const
				{ return mMainThreadLockCount; }

	#endif
	
	#ifdef __ISADORA_APP__
	bool	TryLock(
				IzMilliseconds			inTimeout = 0);
	#endif
	
	#if TARGET_OS_MAC
	inline pthread_mutex_t		GetNativeMutex() { return mMutex; }
	#endif
	
	#if TARGET_OS_WIN
	inline CRITICAL_SECTION		GetNativeMutex() { return mMutex; }
	#endif
	
	#if MUTEX_BACKTRACE_ENABLE
	static void EnableLockUnlockBacktrace(bool inEnable) { sEnableBacktrace = inEnable; }
	#endif
	
private:

	#if TARGET_OS_MAC
		pthread_mutex_t 	mMutex;
	#elif TARGET_OS_WIN
		CRITICAL_SECTION	mMutex;
	#endif
	
	bool		mMutexInitialized;		// true if mutex has been initialized
	SInt32		mLockCount;				// current lock count
	#if MUTEX_KEEP_MAIN_THREAD_LOCK_COUNT
	SInt32		mMainThreadLockCount;	// current lock count within primary thread
	#endif
	bool		mRecursive;				// true if lock is recursive (always true on Windows)
	
	#if MUTEX_BACKTRACE_ENABLE
	static bool	sEnableBacktrace;
	#endif
};

class StMutexLocker
{
public:
	StMutexLocker(
		CMutex&				inMutex) :
			mMutex(inMutex)
	{
		mMutex.Lock();
	}
	
	StMutexLocker(
		const CMutex&		inMutex) :
			mMutex(const_cast<CMutex&>(inMutex))
	{
		mMutex.Lock();
	}
	
	~StMutexLocker()
	{
		mMutex.Unlock();
	}
	
private:
	CMutex&	mMutex;
};
#endif
