// ===========================================================================
//	CPixelSize					(C) 2014 Mark F. Coniglio. All rights reserved
// ===========================================================================

#ifndef _H_CPixelSize
#define _H_CPixelSize

#include "izzy_math.h"
#include "IZGraphics.h"

#ifdef __ISADORA_APP__
#include "LFileStream.h"
#endif

#if TARGET_OS_WIN
#pragma warning( disable : 4068 )
#endif

#pragma mark ----- PIXEL SIZE -----

// ---------------------------------------------------------------------------
// PixelSize

class PixelSize
{
public:

				PixelSize() :
					width(0),
					height(0)
				{
				}
				
				PixelSize(
					SInt32	inWidth,
					SInt32	inHeight) :
						width(inWidth),
						height(inHeight)
				{
				}
				
				#ifdef __ISADORA_APP__
				PixelSize(
					SDimension16		inSize) :
						width(inSize.width),
						height(inSize.height)
				{
				}
				
				PixelSize(
					SDimension32		inSize) :
						width(inSize.width),
						height(inSize.height)
				{
				}

				PixelSize(
					const IzRect&		r) :
						width((r.right - r.left > 0) ? r.right - r.left : 0),
						height((r.bottom - r.top > 0) ? r.bottom - r.top : 0)
				{
				}
				#endif

				PixelSize(
					const CGSize&		inRhs) :
						width((SInt32) inRhs.width),
						height((SInt32) inRhs.height)
				{
				}
				

	#if DELETE_ME
	#if IZ_32BIT && defined(_OBJC_OBJC_H_)
				PixelSize(
					const NSSize&		inRhs) :
						width((SInt32) inRhs.width),
						height((SInt32) inRhs.height)
				{
				}
	#endif
	#endif

	PixelSize&	operator=(
					const CGSize&		inRhs)
				{
					width = (SInt32) inRhs.width;
					height = (SInt32) inRhs.height;
					return *this;
				}

	#if DELETE_ME
	#if IZ_32BIT && defined(_OBJC_OBJC_H_)
	PixelSize&	operator=(
					const NSSize&		inRhs)
				{
					width = (SInt32) inRhs.width;
					height = (SInt32) inRhs.height;
					return *this;
				}

	#endif
	#endif

	#ifdef __ISADORA_APP__
	PixelSize&	operator=(
					SDimension16	inSize)
				{
					width = inSize.width;
					height = inSize.height;
					return *this;
				}

	PixelSize&	operator=(
					SDimension32	inSize)
				{
					width = inSize.width;
					height = inSize.height;
					return *this;
				}
	#endif

	PixelSize	operator*(
					float		inScalingFactor)
				{
					PixelSize ps = *this;
					ps.width = (SInt32)(ps.width * inScalingFactor);
					ps.height = (SInt32)(ps.height * inScalingFactor);
					return ps;
				}

	PixelSize	operator/(
					float		inScalingFactor)
				{
					PixelSize ps = *this;
					ps.width = (SInt32)(ps.width / inScalingFactor);
					ps.height = (SInt32)(ps.height / inScalingFactor);
					return ps;
				}

	PixelSize&	operator*=(
					float		inScalingFactor)
				{
					width = (SInt32)(width * inScalingFactor);
					height = (SInt32)(height * inScalingFactor);
					return *this;
				}

	PixelSize&	operator/=(
					float		inScalingFactor)
				{
					width = (SInt32)(width / inScalingFactor);
					height = (SInt32)(height / inScalingFactor);
					return *this;
				}

	bool		operator==(
					const PixelSize&	inRhs) const
				{
					return inRhs.width == width && inRhs.height == height;
				}

	bool		operator!=(
					const PixelSize&	inRhs) const
				{
					return inRhs.width != width || inRhs.height != height;
				}

	bool		IsValid() const
					{ return (width > 0 && height > 0); }
	
	void		Reset()
					{ width = height = 0; }
					
	float		GetAspect() const
				{
					if (height > 0) {
						return float(width) / float(height);
					} else {
						return 0.0f;
					}
				}
	
	/*
	void		Scale(
					float		inScaleFactor)
				{
					width = SInt32(round(width * inScaleFactor));
					height = SInt32(round(height * inScaleFactor));
				}
	
	void		Scale(
					float		inScaleFactorX,
					float		inScaleFactorY)
				{
					width = SInt32(round(width * inScaleFactorX));
					height = SInt32(round(height * inScaleFactorY));
				}
	*/

	PixelSize	GetScaledSize(
					float		inScaleFactor) const
				{
					return PixelSize((UInt32) round(width * inScaleFactor), (UInt32) round(height * inScaleFactor));
				}
	
	PixelSize	GetScaledSize(
					float		inScaleFactorX,
					float		inScaleFactorY) const
				{
					return PixelSize((UInt32) round(width * inScaleFactorX), (UInt32) round(height * inScaleFactorY));
				}

	PixelSize	GetLimited(
					SInt32		inMinWidth,						// MIN WIDTH
					SInt32		inMinHeight,					// MIN HEIGHT
					SInt32		inMaxWidth = 0x7FFFFFFF,		// MAX WIDTH
					SInt32		inMaxHeight = 0x7FFFFFFF)		// MAX HEIGHT
				{
					PixelSize size = *this;
					size.Limit(inMinWidth, inMinHeight, inMaxWidth, inMaxHeight);
					return size;
				}
	
	void		Limit(
					SInt32		inMinWidth,						// MIN WIDTH
					SInt32		inMinHeight,					// MIN HEIGHT
					SInt32		inMaxWidth = 0x7FFFFFFF,		// MAX WIDTH
					SInt32		inMaxHeight = 0x7FFFFFFF)		// MAX HEIGHT
				{
					if (width < inMinWidth) {
						width = inMinWidth;
					} else if (width > inMaxWidth) {
						width = inMaxWidth;
					}
					if (height < inMinHeight) {
						height = inMinHeight;
					} else if (height > inMaxHeight) {
						height = inMaxHeight;
					}
				}
	
	void		Limit(
					const PixelSize&	inMin,
					const PixelSize&	inMax)
				{
					if (width < inMin.width) {
						width = inMin.width;
					} else if (width > inMax.width) {
						width = inMax.width;
					}
					if (height < inMin.height) {
						height = inMin.height;
					} else if (height > inMax.height) {
						height = inMax.height;
					}
				}
	
	SInt32		GetArea() const
					{ return width * height; }

	static IzRect				PreserveAspectRatioWithinRect(
									PixelSize		inSrcSize,
									const IzRect&		inDstRect);
	
	static PixelSize			CalculatePreservedAspectRatio(
									PixelSize		inSrcSize,
									PixelSize		inDstSize);
	
	SInt32	width;
	SInt32	height;
};


// ---------------------------------------------------------------------------
// PixelSize

#ifdef __ISADORA_APP__
class PixelSizeIO : public PixelSize
{
public:
	enum { class_ID = FOUR_CHAR_CODE('PXsz') };

						PixelSizeIO() :
							PixelSize()
						{
						}
						
						PixelSizeIO(
							UInt32	inWidth,
							UInt32	inHeight) :
								PixelSize(inWidth, inHeight)
						{
						}

						PixelSizeIO(
							const PixelSize&	inPixelSize) :
								PixelSize(inPixelSize)
						{
						}

						PixelSizeIO(
							LStream*	inStream);
							
	UInt32				Write(
							LStream*	inStream) const;
	
	UInt32				Read(
							LStream*	inStream);
};
#endif

#pragma mark ----- PIXEL POS -----

// ---------------------------------------------------------------------------
// PixelPos

class PixelPos
{
public:

				PixelPos() :
					x(0),
					y(0)
				{
				}
				
				PixelPos(
					SInt32	inX,
					SInt32	inY) :
						x(inX),
						y(inY)
				{
				}

	
	PixelPos	operator +(
					const PixelPos&	inValue) const
						{ return PixelPos(x + inValue.x, y + inValue.y); }
						
	PixelPos&	operator +=(
					const PixelPos&	inValue)
				{
					x += inValue.x;
					y += inValue.y;
					return (*this);
				}
	
	PixelPos	operator -(
					const PixelPos&	inValue) const
						{ return PixelPos(x - inValue.x, y - inValue.y); }
						
	PixelPos&	operator -=(
					const PixelPos&	inValue)
				{
					x -= inValue.x;
					y -= inValue.y;
					return (*this);
				}

	bool		operator==(
					const PixelPos&	inRhs) const
				{
					return inRhs.x == x && inRhs.y == y;
				}

	bool		operator!=(
					const PixelPos&	inRhs) const
				{
					return inRhs.x != x || inRhs.y != y;
				}

	void		Reset()
					{ x = y = 0; }
					
	SInt32	x;
	SInt32	y;
};

// ---------------------------------------------------------------------------
// PixelPos

#ifdef __ISADORA_APP__
class PixelPosIO : public PixelPos
{
public:
	enum { class_ID = FOUR_CHAR_CODE('PXps') };
	
						PixelPosIO() :
							PixelPos()
						{
						}
	
						PixelPosIO(
							const PixelPos&		inPos)
								: PixelPos(inPos)
						{
						}
						
						PixelPosIO(
							SInt32	inLeft,
							SInt32	inTop) :
								PixelPos(inLeft, inTop)
						{
						}

						PixelPosIO(
							LStream*	inStream);
							
	UInt32				Write(
							LStream*	inStream) const;
	
	UInt32				Read(
							LStream*	inStream);
};
#endif


#endif // _H_CPixelSize
