// ===========================================================================
//	CVideoInputMixin	   (c) 2012 Mark F. Coniglio. All rights reserved.
// ===========================================================================

#ifndef _H_CVideoInputMixin
#define	_H_CVideoInputMixin
#pragma once

#include "IsadoraTypes.h"
#include "ImageBufferCommon.h"
#include "CoreImageBase.h"
#include "TextureBufferCommon.h"
#include "CPixelSize.h"
#include "CFloatXY.h"

// ---------------------------------------------------------------------------------

class CVideoInputMixin
{
public:
							CVideoInputMixin();
		
	virtual					~CVideoInputMixin();
	
	void					ActivateVideoInputMixin();
	
	void					DeactivateVideoInputMixin();
	
	virtual void			SetInputFrame(
								ValuePtr		inNewValue,
								StageIndexT		inBitmapTargetStageIndex0);
	
	virtual void			ClearAllInputFrames();

	PixelSize				GetInputFrameSize(
								bool*	outFrameSizeChanged = NULL);

	void					UpdatePendingStageIndexWithOverride();
	
// STAGE INDEX
	inline bool				StageIndexNeedsUpdate() const
								{ return mPendingStageIndex0 != mCurrentStageIndex0; }
								
	inline void				UpdateCurrentStageIndex()
								{ mCurrentStageIndex0 = mPendingStageIndex0; }
								
	inline StageIndexT		GetCurrentStageIndex0() const
								{ return mCurrentStageIndex0; }
								
	inline StageIndexT		GetPendingStageIndex0() const
								{ return mPendingStageIndex0; }
								
	inline StageIndexT		GetOverrideStageIndex0() const
								{ return mOverrideStageIndex0; }
	
	inline void				SetPendingStageIndex0(
								StageIndexT		inStageIndex0)
									{ mPendingStageIndex0 = inStageIndex0; }
	
	inline void				SetOverrideStageIndex(
								StageIndexT			inOverrideStageIndex0)
							{
								mOverrideStageIndex0 = inOverrideStageIndex0;
								mPendingStageIndex0 = inOverrideStageIndex0;
							}
								
	inline void				ClearOverrideStageIndex()
								{ mOverrideStageIndex0 = kInvalidStageIndex; }
								
	inline void				ResetAllTargetStageIndexes()
							{
								mPendingStageIndex0 = kInvalidStageIndex;
								mCurrentStageIndex0 = kInvalidStageIndex;
								mOverrideStageIndex0 = kInvalidStageIndex;
							}
							
// DATA TYPES
	inline bool				DataTypeNeedsUpdate() const
								{ return mPendingDataType != mCurrentDataType; }
								
	inline void				UpdateCurrentDataType()
								{ mCurrentDataType = mPendingDataType; }

	inline DataType			GetCurrentDataType() const
								{ return mCurrentDataType; }

	inline DataType			GetPendingDataType() const
								{ return mPendingDataType; }

								
// IMAGE BUFFER INPUT
	inline bool				HasValidBitmapImageInput() const
								{ return mSourceBitmapBuf != NULL && mSourceBitmapBuf->mMagic == kImageBufferMagic; }

	ImageBufferPtr			GetBitmapImageInput() const
							{
								if (HasValidBitmapImageInput()) {
									return mSourceBitmapBuf;
								} else {
									return NULL;
								}
							}
							
// TEXTURE BUFFER INPUT
	inline bool				HasValidTextureInput() const
							{
								return mSourceTextureBuf != NULL && mSourceTextureBuf->mTexture != NULL && mSourceTextureBuf->mMagic == kTextureBufferMagic;
							}

	TextureBufferPtr		GetTextureImageInput() const
							{
								return mSourceTextureBuf;
							}
							
// CORE IMAGE BUFFER INPUT
	inline bool				HasValidCoreImageInput() const
							{
								#if COMPILE_CORE_VIDEO
									return mSourceCoreImageBuf != NULL && mSourceCoreImageBuf->mImage != NULL && mSourceCoreImageBuf->mMagic == kCoreImageBufferMagic;
								#else
									return false;
								#endif
							}
							
	#if COMPILE_CORE_VIDEO

	CoreImageBufferPtr		GetCoreImageInput() const
							{
								if (HasValidCoreImageInput()) {
									return mSourceCoreImageBuf;
								} else {
									return NULL;
								}
							}
							
	CoreImageBufferPtr		GetCoreImageInputWithModulatedAlpha(
								float			inAlphaModulation_0to1);
	

	CoreImageBufferPtr		GetCoreImageInputWithModulatedARGB(
								float			inModulation_0to1);
	
	#endif

	void					ReleaseCoreImageColorMatrixImage();
	
	bool					UpdateResolutionChangedFlag();
	
private:
// VIDEO INPUT BUFFERS
	TextureBufferPtr			mSourceTextureBuf;
	IzzyTexture					mSourceTexture;			// retained/released by us
	
	ImageBufferPtr				mSourceBitmapBuf;
	
	#if COMPILE_CORE_VIDEO
	CoreImageBufferPtr			mSourceCoreImageBuf;
	CI_Image					mSourceCIImage;			// retained/released by us
	CoreImageBuffer				mColorMatrixBuffer;		// created/retained/released by ys
	CI_ColorMatrixFilter		mColorMatrixFilter;		// created/retained/released by us
	#endif

// VIDEO INPUT DATA TYPE
	DataType					mPendingDataType;
	DataType					mCurrentDataType;

// STAGE INDEXES
	StageIndexT					mInputStreamStageIndex0;	// stage index as determined by input stream
	StageIndexT					mPendingStageIndex0;		// stage index waiting to made current by UpdateCurrentStageIndex()
	StageIndexT					mCurrentStageIndex0;		// current stage - made current by calling UpdateCurrentStageIndex()
	StageIndexT					mOverrideStageIndex0;		// if not kInvalidStageIndex, then force this stage index

// RESOLUTION
	PixelSize					mLastResolution;
};
#endif

typedef CVideoInputMixin*	CVideoInputMixinPtr;
