// ===========================================================================
//	IZTypes
// ===========================================================================

#ifndef _H_IZQT
#include "IZQT.h"
#endif

#ifndef _H_IZTypes
#define _H_IZTypes

#include <float.h>
#include <stdint.h>

// ---------------------------------------------------------------------------------
//	TYPEDEFS
// ---------------------------------------------------------------------------------
#define kIzPointZero	IzPoint_00

#define kIzRectZero		IzRect_0000

// IzInt TYPE
//
// The size of this integer value will vary between
// platforms, being a 8 bytes for 64 bit targets, and
// 4 bytes for 32 bit targets. For conistent integer
// sizes, use IzInt32, IzUInt32, IzInt64, IzUInt64
//
// Note that the ivalue member of the Value struct is
// guaranteed to have the same type and size as IzInt

// IzReal TYPE
//
// the size of this floating point value will be 8
// bytes for 64 bit targets, and 4 bytes for 32 bit
// targets.
//
// Note that the fvalue member of the Value struct is
// guaranteed to have the same type and size as IzReal

// IzFloat32 TYPE
//
// IzFloat: the size of this floating point value will be
// 4 bytes all targets
//
#if IZ_32BIT

	typedef SInt32				IzInt;				// 4 bytes
	#define IZ_INT_MAX			INT32_MAX
	#define IZ_INT_MIN			INT32_MIN
	#define	IZ_INT_SIZE			4

	typedef UInt32				IzUInt;				// 8 bytes
	#define IZ_UINT_MAX			UINT32_MAX
	#define IZ_UINT_MIN			0
	#define	IZ_UINT_SIZE		4

	typedef float				IzReal;				// 4 bytes
	#define IZ_REAL_MAX			+FLT_MAX
	#define IZ_REAL_MIN			-FLT_MAX
	#define	IZ_REAL_SIZE		4

	typedef void*				IzPtr;				// 4 bytes
	#define	IZ_PTR_SIZE			4

#else

	typedef int64_t				IzInt;				// 8 bytes
	#define IZ_INT_MAX			INT64_MAX
	#define IZ_INT_MIN			INT64_MIN
	#define	IZ_INT_SIZE			8

	typedef uint64_t			IzUInt;				// 8 bytes
	#define IZ_UINT_MAX			UINT64_MAX
	#define IZ_UINT_MIN			0
	#define	IZ_UINT_SIZE		8

	typedef double				IzReal;				// 8 bytes
	#define IZ_REAL_MAX			+DBL_MAX
	#define IZ_REAL_MIN			-DBL_MAX
	#define	IZ_REAL_SIZE		8

	typedef void*				IzPtr;				// 8 bytes
	#define	IZ_PTR_SIZE			8
	
#endif

typedef float			IzFloat32;			// 4 bytes
#define IZ_FLT32_MAX	+FLT_MAX
#define IZ_FLT32_MIN	-FLT_MAX
#define	IZ_FLOAT32_SIZE	4

typedef int64_t			IzInt64;
#define	IZ_INT64_MAX	INT64_MAX
#define	IZ_INT64_MIN	INT64_MIN
#define IZ_INT64_SIZE	8

typedef uint64_t		IzUInt64;
#define	IZ_UINT64_MAX	UINT64_MAX
#define	IZ_UINT64_MIN	0
#define IZ_UINT64_SIZE	8

typedef int32_t			IzInt32;
#define	IZ_INT32_MAX	INT32_MAX
#define	IZ_INT32_MIN	INT32_MIN
#define IZ_INT32_SIZE	4

typedef uint32_t		IzUInt32;
#define	IZ_UINT32_MAX	UINT32_MAX
#define	IZ_UINT32_MIN	0
#define IZ_UINT32_SIZE	4

typedef int16_t			IzInt16;
#define	IZ_INT16_MAX	INT16_MAX
#define	IZ_INT16_MIN	INT16_MIN
#define IZ_INT16_SIZE	2

typedef uint16_t		IzUInt16;
#define	IZ_UINT16_MAX	UINT16_MAX
#define	IZ_UINT16_MIN	0
#define IZ_UINT16_SIZE	2

typedef SInt16			IzResID;

#if IZ_64BIT
	typedef void*		IzRefCon;
#elif IZ_32BIT
	typedef SInt32		IzRefCon;
#endif

#ifdef __cplusplus

#if IZ_64BIT

	typedef int64_t	IzRefConInt;

	inline IzRefCon		PtrToIzRefCon(void* v)			{ return reinterpret_cast<IzRefCon>(v); }
	inline IzRefCon		IntToIzRefCon(IzRefConInt v)	{ return reinterpret_cast<IzRefCon>(v); }
	inline IzRefCon		RealToIzRefCon(IzReal v)		{ return reinterpret_cast<IzRefCon>(*((void**)&v)); }
	inline void*		IzRefConToPtr(IzRefCon r)		{ return reinterpret_cast<void*>(r); }
	inline IzRefConInt	IzRefConToInt(IzRefCon r)		{ return reinterpret_cast<IzRefConInt>(r); }
	inline IzReal		IzRefConToReal(IzRefCon r)		{ return *((IzReal*)&r); }
	inline IzReal*		IzRefConToRealPtr(IzRefCon r)	{ return reinterpret_cast<IzReal*>(r); }

	#define IzRefConZero	IzRefCon(NULL)

	#if TARGET_OS_WIN
	typedef double	CGFloat;
	#endif

#else

	typedef SInt32	IzRefConInt;

	inline IzRefCon		PtrToIzRefCon(void* v)			{ return reinterpret_cast<IzRefCon>(v); }
	inline IzRefCon		IntToIzRefCon(IzRefConInt v)	{ return v; }
	inline IzRefCon		RealToIzRefCon(IzReal v)		{ return *((IzRefConInt*)&v); }
	inline void*		IzRefConToPtr(IzRefCon r)		{ return reinterpret_cast<void*>(r); }
	inline IzRefConInt	IzRefConToInt(IzRefCon r)		{ return r; }
	inline IzReal		IzRefConToReal(IzRefCon r)		{ return *((float*)&r); }
	inline IzReal*		IzRefConToRealPtr(IzRefCon r)	{ return ((float*)&r); }

	#define IzRefConZero	IzRefCon(0)

	#if TARGET_OS_WIN
	typedef float	CGFloat;
	#endif

#endif

#endif

// ---------------------------------------------------------------------------------
//	WINDOWS COMPATIBILITY
// ---------------------------------------------------------------------------------
// Add these typedefs so that they are valid on MacOS as well as Windows

#if TARGET_OS_MAC

#define	WINAPI

typedef	UInt8		BYTE;
typedef SInt16		SHORT;
typedef UInt16		USHORT;
typedef	SInt32		INT;
typedef	UInt32		UINT;
typedef SInt32		LONG;
typedef UInt32		ULONG;
typedef UInt32		DWORD;
typedef SInt64		LONGLONG;
typedef UInt64		ULONGLONG;

typedef DWORD		COLORREF;
 
typedef void*		LPVOID;

typedef char*		LPSTR;
typedef const char*	LPCSTR;

// N.B. WCHAR as defined here for macOS **CANNOT** be
// used interchangably with wchar_t.
//
// On Windows, WCHAR is defined a wchar_t which is a
// two-byte unsigned integer (i.e., UTF-16). To ensure
// crossplatform compatibility we define WCHAR here to
// match that definition. Importantly, this ensure that
// the underlying type for WCHAR matches IzUniChar,
// which is also UTF-16.
//
// In terms of cross-platform compilation this creates
// a discrepency: attempting to assign a WCHAR string
// a std::wstring will fail to compile on macOS because
// WCHAR is not the same as wchar_t. On Windows, such
// an assignment will compile successfully.

typedef unsigned short		WCHAR;
 
typedef void*		HINSTANCE;
typedef void*		HWND;
typedef void*		HDC;
typedef	void*		HGLRC;

typedef struct tagPOINT {
  LONG x;
  LONG y;
} POINT, *PPOINT;

typedef struct tagSIZE {
  LONG cx;
  LONG cy;
} SIZE, *PSIZE;

typedef struct _RECT {
  LONG left;
  LONG top;
  LONG right;
  LONG bottom;
} RECT, *PRECT;

#endif

#endif	// #if _H_IZDefinitions
