//============================================================================
//	IzzyFrameBufferBase			(c) 2015 Mark F. Coniglio. All rights reserved
// ===========================================================================

#ifndef _H_IzzyFrameBufferBase
#define	_H_IzzyFrameBufferBase
#pragma once

// system includes
#include "OpenGLIncludes.h"
#include "CPlusPlusMacros.h"
#include "CGLFunctions.h"

// isadora includes
#include "IzzyTextureBase.h"

#if defined(__cplusplus) && defined(__ISADORA_APP__)
	class								CGLFrameBuffer;
	class								CGLTexture;
#else
	typedef struct OpaqueCGLTexture*	CGLFrameBuffer;
	typedef struct OpaqueCGLTexture*	CGLTexture;
#endif

#ifdef __ISADORA_APP__
	#define	IZZYFRAMEBUFFER_ASSERT_ON_BAD_GLFRAMEBUFFER		(TRUE && DEBUG)
#else
	#define	IZZYFRAMEBUFFER_ASSERT_ON_BAD_GLFRAMEBUFFER		FALSE
#endif

// ===========================================================================

// v2.6 ->
enum FBOBlendMode {
	kFBOBlendMode_Additive,
	kFBOBlendMode_Transparent,
	kFBOBlendMode_Opaque
};

enum FBUnpremultiplyMode {
	kFBUnpremultiplyMode_No,
	kFBUnpremultiplyMode_Yes
};
// v2.6 <-

// ===========================================================================
// IsadoraFrameBuffer struct

// VIEWPORT NOTE: the mFBOWidth and mFBOHeight members
// define the viewport dimensions
//
// For Isadora FBOs, the viewport will always be set with
// the call glViewport(0, 0, mFBOWidth, mFBOHeight);

CLASSHDR IsadoraFrameBuffer
{
PUBLIC

	#ifdef __cplusplus
	IsadoraFrameBuffer(
		CIZGLFunctions*	inGLBufferFunctions,
		CGLFrameBuffer*	inGLBufferOwner,
		IzzyTexture		inFBOTexture = NULL,
		GLuint			inFBOFrameBufferID = 0,
		GLuint			inFBODepthRenderBufferID = 0,
		GLuint			inFBOWidth = 0,
		GLuint			inFBOHeight = 0) :
			mFBOGLFunctions(inGLBufferFunctions),
			mFBOFrameBufferOwner(inGLBufferOwner),
			mFBOTexture(inFBOTexture),
			mFBOFrameBufferID(inFBOFrameBufferID),
			mFBODepthRenderBufferID(inFBODepthRenderBufferID),
			mFBOWidth(inFBOWidth),
			mFBOHeight(inFBOHeight),
			mFBOIsBound(GL_FALSE),
			mFBOCIContext(NULL)
	{
	}

	inline bool				FBO_HasCGLFrameBuffer() const		{ return mFBOFrameBufferOwner != NULL; }

#if IZZYFRAMEBUFFER_ASSERT_ON_BAD_GLFRAMEBUFFER
	CGLFrameBuffer*			__FBO_CGLFrameBuffer(const char* inFile, int inLine) const;
#else
	inline CGLFrameBuffer*	FBO_CGLFrameBuffer() const			{ return mFBOFrameBufferOwner; }
#endif
	
	inline bool				FBO_HasTexture() const				{ return mFBOTexture != NULL; }
	inline void				FBO_ClearTexture()					{ mFBOTexture = NULL; }

	inline CGLTexture*		FBO_CGLTexture() const				{ return (mFBOTexture != NULL) ? mFBOTexture->GetCGLTexture() : NULL; }
	inline IzzyTexture		FBO_IzzyTexture() const				{ return mFBOTexture; }

	inline GLuint			FBO_FrameBufferID() const			{ return mFBOFrameBufferID; }
	inline GLuint			FBO_DepthRenderBufferID() const		{ return mFBODepthRenderBufferID; }
	inline GLuint			FBO_Width() const					{ return mFBOWidth; }
	inline GLuint			FBO_Height() const					{ return mFBOHeight; }
	
	inline GLboolean		FBO_IsBound() const					{ return mFBOIsBound; }
	
	inline void				FBO_SetCIContext(void* inCtx)		{ mFBOCIContext = inCtx; }
	inline void				FBO_ClearCIContext()				{ mFBOCIContext = NULL; }
	inline void*			FBO_GetCIContext()					{ return mFBOCIContext; }
	#endif

PUBLIC
	CIZGLFunctions*			mFBOGLFunctions;
PROTECTED
	CGLFrameBuffer*			mFBOFrameBufferOwner;
	IzzyTexture				mFBOTexture;
	GLuint					mFBOFrameBufferID;
	GLuint					mFBODepthRenderBufferID;
	GLuint					mFBOWidth;
	GLuint					mFBOHeight;
	GLuint					mFBOIsBound;
	void*					mFBOCIContext;						// MAC OS X ONLY
};

typedef IsadoraFrameBuffer* IzzyFrameBuffer;

typedef IzzyFrameBuffer* IzzyFrameBufferPtr;

#if IZZYFRAMEBUFFER_ASSERT_ON_BAD_GLFRAMEBUFFER

	#define FBO_CGLFrameBuffer()	__FBO_CGLFrameBuffer(__FILE__, __LINE__)

#endif

#endif	// #ifndef _H_IzzyFrameBufferBase
