//============================================================================
//	IzzyTextureBase				(c) 2015 Mark F. Coniglio. All rights reserved
// ===========================================================================

#ifndef _H_IzzyTextureBase
#define	_H_IzzyTextureBase
#pragma once

// system includes
#include "OpenGLIncludes.h"
#include "CPlusPlusMacros.h"

#if defined(__cplusplus) && defined(__ISADORA_APP__)
	class								CGLTexture;
#else
	typedef struct OpaqueCGLTexture*	CGLTexture;
#endif

#ifdef __ISADORA_APP__
	#define	IZZYTEXTURE_ASSERT_ON_BAD_GLTEXTURE		(TRUE && DEBUG)
#else
	#define	IZZYTEXTURE_ASSERT_ON_BAD_GLTEXTURE		FALSE
#endif

// ===========================================================================
// IsadoraTexture struct

CLASSHDR IsadoraTexture
{
PUBLIC

	#ifdef __cplusplus
						IsadoraTexture(
							CGLTexture*	inGLTextureOwner) :
								mRetainCount(1),
								mGLTextureOwner(inGLTextureOwner),
								mTextureName(0),
								mTextureTarget(0),
								mTextureGPUWidth(0),
								mTextureGPUHeight(0),
								mTextureVisibleWidth(0),
								mTextureVisibleHeight(0),
								mTextureClampModeCurrent(GL_TEXTURE_WRAP_S),		// current texture clamp mode
								mTextureClampModePending(GL_TEXTURE_WRAP_R),		// pending texture clamp mode
								mTextureIsPremultiplied(GL_FALSE),
								mTextureUniqueID(0),
								mMipmapEnabled(GL_FALSE),
								mMipmapDirty(GL_FALSE)
						{
						}

						IsadoraTexture(
							CGLTexture*	inGLTextureOwner,
							GLuint		inTextureName,
							GLenum		inTextureTarget,
							GLuint		inTextureGPUWidth,
							GLuint		inTextureGPUHeight,
							GLuint		inTextureVisibleWidth,
							GLuint		inTextureVisibleHeight,
							GLuint		inTextureIsPremultiplied = GL_FALSE,
							GLuint		inTextureUniqueID = 0) :
								mRetainCount(1),
								mGLTextureOwner(inGLTextureOwner),
								mTextureName(inTextureName),
								mTextureTarget(inTextureTarget),
								mTextureGPUWidth(inTextureGPUWidth),
								mTextureGPUHeight(inTextureGPUHeight),
								mTextureVisibleWidth(inTextureVisibleWidth),
								mTextureVisibleHeight(inTextureVisibleHeight),
								mTextureClampModeCurrent(GL_TEXTURE_WRAP_S),		// current texture clamp mode
								mTextureClampModePending(GL_TEXTURE_WRAP_R),		// pending texture clamp mode
								mTextureIsPremultiplied(inTextureIsPremultiplied),
								mTextureUniqueID(inTextureUniqueID),
								mMipmapEnabled(GL_FALSE),
								mMipmapDirty(GL_FALSE)
						{
						}

protected:
						virtual ~IsadoraTexture()
						{
							#ifdef __ISADORA_APP__
							DebugAssert_(mRetainCount == 0);
							DebugAssert_(mTextureUniqueID != 0xDEADDEAD);
							#endif
							
							mGLTextureOwner = NULL;
							mTextureName = 0;
							mTextureTarget = 0;
							mTextureGPUWidth = mTextureGPUHeight = 0;
							mTextureVisibleWidth = mTextureVisibleHeight = 0;
							mTextureIsPremultiplied = 0;
							mTextureUniqueID = 0;
							mMipmapEnabled = GL_FALSE;
							mMipmapDirty = GL_FALSE;


							mTextureUniqueID = 0xDEADDEAD;
						}

public:

	virtual void		Retain()	
							{ mRetainCount += 1; }
							
	virtual void		Release()
						{
							#ifdef __ISADORA_APP__
							DebugAssert_(mRetainCount > 0);
							DebugAssert_(mTextureUniqueID != 0xDEADDEAD);
							#endif
							
							if (--mRetainCount == 0) {

								#ifdef __ISADORA_APP__
								#if FALSE && DEBUG
									log_info("Delete IzzyTexture %p\n", this);
								#endif
								#endif

								delete this;
							}
						}
	
	size_t				GetRetainCount() { return mRetainCount; }
	
	virtual void		RetainOwningContext() { }

	virtual void		ReleaseOwningContext() { }

	virtual void		RetainTextureAndContext()
						{
							Retain();
						}
							
	virtual void		ReleaseTextureAndContext()
						{
							Release();
						}
	
	size_t				TextureGetRetainCount() const						{ return mRetainCount; }

	inline bool			IsCGLTextureValid() const							{ return mGLTextureOwner != NULL; }
	
	#if IZZYTEXTURE_ASSERT_ON_BAD_GLTEXTURE
	CGLTexture*			__GetCGLTexture(const char* inFile, int inLine) const;
	#else
	inline CGLTexture*	GetCGLTexture() const								{ return mGLTextureOwner; }
	#endif
	
	inline GLuint		GetTextureName() const								{ return mTextureName; }
	inline GLenum		GetTextureTarget() const							{ return mTextureTarget; }
	inline GLuint		GetTextureGPUWidth() const							{ return mTextureGPUWidth; }
	inline GLuint		GetTextureGPUHeight() const							{ return mTextureGPUHeight; }
	inline GLuint		GetTextureVisibleWidth() const						{ return mTextureVisibleWidth; }
	inline GLuint		GetTextureVisibleHeight() const						{ return mTextureVisibleHeight; }
	inline GLint		GetTextureClampMode() const							{ return mTextureClampModeCurrent; }
	inline GLuint		GetTextureIsPremultiplied() const					{ return mTextureIsPremultiplied; }
	inline void			SetTextureIsPremultiplied(GLuint isPremultiplied)	{ mTextureIsPremultiplied = isPremultiplied; }
	inline GLuint		GetTextureUniqueID() const							{ return mTextureUniqueID; }
	
	inline bool			GetTextureMipmapEnabled() const						{ return (mMipmapEnabled != GL_FALSE); }
	
	inline bool			SetTextureMipmapEnabled(
							bool			inMipmapEnabled)
						{
							bool mipmapBecameDirty = false;
							if (!mMipmapEnabled && inMipmapEnabled) {
								mMipmapEnabled = mMipmapDirty = GL_TRUE;
								mipmapBecameDirty = GL_TRUE;
							} else if (mMipmapEnabled && !inMipmapEnabled) {
								mMipmapEnabled = GL_FALSE;
								mMipmapDirty = GL_FALSE;
							}
							return mipmapBecameDirty;
						}
	
	inline bool			GetTextureMipmapDirty() const						{ return (mMipmapEnabled && mMipmapDirty); }
	inline void			SetTextureMipmapDirty()								{ mMipmapDirty = GL_TRUE; }
	inline void			ClearTextureMipmapDirty()							{ mMipmapDirty = GL_FALSE; }
	
	inline GLint		ChangeTextureClampMode(
							GLint			inNewTextureClampMode)
						{
							GLint oldPendingTextureClampMode = mTextureClampModePending;
							mTextureClampModePending = inNewTextureClampMode;
							return oldPendingTextureClampMode;
						}
						
	inline void			ForceTextureClampModeCurrent()
						{
							glTexParameteri(mTextureTarget, GL_TEXTURE_WRAP_S, mTextureClampModePending);
							glTexParameteri(mTextureTarget, GL_TEXTURE_WRAP_T, mTextureClampModePending);
							mTextureClampModeCurrent = mTextureClampModePending;
						}

	inline void			MakeTextureClampModeCurrent()
						{
							if (mTextureClampModePending != mTextureClampModeCurrent) {
								ForceTextureClampModeCurrent();
							}
						}

	inline void			GetTextureCoordinateExtent(
							float&	outHorz,
							float&	outVert)
						{
							if (mTextureTarget == GL_TEXTURE_RECTANGLE_ARB) {
								outHorz = float(mTextureVisibleWidth);
								outVert = float(mTextureVisibleHeight);
							} else {
								outHorz = float(mTextureVisibleWidth) / float(mTextureGPUWidth);
								outVert = float(mTextureVisibleHeight) / float(mTextureGPUHeight);
							}
						}
						
	inline GLuint		GetTextureFrameNumber() const						{ return mTextureFrameNumber; }
	inline void			SetTextureFrameNumber(GLuint inFrameNumber)			{ mTextureFrameNumber = inFrameNumber; }
	
	#endif

PROTECTED
	size_t				mRetainCount;					// retain count for this object
	CGLTexture*			mGLTextureOwner;				// private to the Isadora Application
	GLuint				mTextureName;					// the texture name
	GLenum				mTextureTarget;					// either GL_TEXTURE_2D, GL_TEXTURE_RECTANGLE_ARB, or GL_TEXTURE_CUBEMAP
	GLuint				mTextureGPUWidth; 				// texture height on GPU
	GLuint				mTextureGPUHeight; 				// texture width on GPU
	GLuint				mTextureVisibleWidth;			// texture visible width
	GLuint				mTextureVisibleHeight;			// texture visible height
	GLint				mTextureClampModeCurrent;		// current texture clamp mode
	GLint				mTextureClampModePending;		// pending texture clamp mode
	GLuint				mTextureIsPremultiplied;		// non-zero if texture is premultiplied
	GLuint				mTextureUniqueID;				// unique ID assigned at creation
	GLboolean			mMipmapEnabled;					// true if mipmaps enabled for this texture
	GLboolean			mMipmapDirty;					// true if mipmaps must be generated for this texture
	GLuint				mTextureFrameNumber;			// used for debugging purposes
};

typedef IsadoraTexture* IzzyTexture;

#if IZZYTEXTURE_ASSERT_ON_BAD_GLTEXTURE

	#define GetCGLTexture()	__GetCGLTexture(__FILE__, __LINE__)

#endif

#endif	// #ifndef _H_IzzyTextureBase
