// ===========================================================================
// IzzyThreads					(c) 2006 Mark F. Coniglio.  all rights reserved.
// ===========================================================================
// Cross platform threading for Mac OS X and Windows

#ifndef _H_IzzyThreads
#define _H_IzzyThreads

#include "ThreadUtils.h"

class CMutex;

#ifdef __cplusplus
extern "C" {
#endif

#define	IZZY_USE_THREADS		1

	enum IzzyThreadErr {
	
		IT_NoError						= 0,
		
		IT_UnknownError					= -1,
		
		IT_ThreadCantCreate				= -2,
		IT_ThreadInvalidAttribute		= -3,
		IT_ThreadInsufficientPrivileges	= -4,
		IT_ThreadNotFound				= -5,
		IT_ThreadNotJoinable			= -6,
		IT_ThreadDeadlock				= -7,
		IT_ThreadCantDetach				= -8,

		IT_ConditionCantCreate			= -9,
		IT_ConditionOutOfMemory			= -10,
		IT_ConditionBusy				= -11,
		IT_ConditionInvalidAttribute	= -12,

		IT_MutexCantCreate				= -13,
		IT_MutexOutOfMemory				= -14,
		IT_MutexBusy					= -15,
		IT_MutexInvalidAttribute		= -16,
		IT_MutexInsufficientPrivileges	= -17
	};
	
	enum IzzyThreadDisposeMode {
		kOwnerDispose = 0,

		kAutoDispose,					// automatically call IzzyDisposeThread() on the IzzyThreadInfo
										// pointed upon completion of the main function
										 
		kAutoDispose_FreeThreadInfo		// in addition to the above, dispose the IzzyThreadInfo pointer using free()
	};
	
	// constant used by some plugins
	const int			IZZY_MAX_THREADS = 2;
	const int			MAX_THREAD_NAME_SIZE = 64;

	// define the localized type for a thread ID
	#if TARGET_OS_MAC
		typedef	pthread_t	IzzyThreadID;
		#define				IsValidThreadID(id)		((id) != NULL)
	#elif TARGET_OS_WIN
		typedef DWORD		IzzyThreadID;
		#define				IsValidThreadID(id)		((id) != 0)
	#endif

	// typedef for the thread execute function
	typedef int				(*IzzyThreadExecuteFunc)(IzzyThreadID inThreadID, void* inUserData);
	
	// typedef for the IzzyThreadInfo struct
	typedef struct {
		IzzyThreadExecuteFunc	mThreadFunc;
		void*					mUserData;
		IzzyThreadID			mThreadID;
		bool					mCanSignal;
		
		IzzyThreadDisposeMode	mAutoDisposeMode;
		
		#if TARGET_OS_MAC
		
			bool					mThreadDetatched;
			bool					mThreadJoined;
			
			pthread_cond_t			mCondition;
			bool					mConditionValid;
			
			pthread_mutex_t			mCondMutex;
			bool					mMutexValid;
			bool					mOwnsMutex;
			
		#elif TARGET_OS_WIN
			HANDLE					mThreadHandle;
			HANDLE					mCondition;
		#endif
		
		char						mThreadName[MAX_THREAD_NAME_SIZE];
	
	} IzzyThreadInfo;
	
	enum IzzyThreadPriority {
		kIzzyThreadPriority_Low = -2,
		kIzzyThreadPriority_BelowNormal = -1,
		kIzzyThreadPriority_Normal = 0,
		kIzzyThreadPriority_AboveNormal = 1,
		kIzzyThreadPriority_High = 2
	};
	
	bool			IsIzzyThreadActive(
						const IzzyThreadInfo*		inThreadInfo);
		
	// creates a new thread and immediately starts processing
	IzzyThreadErr	IzzyCreateAndRunThread(
						IzzyThreadExecuteFunc	inFunction,
						void*					inUserData,
						IzzyThreadDisposeMode	inAutoDisposeMode,
						bool					inRequiresSignal,
						CMutex*					inMutexObject,		// CAN BE NULL
						const char*				inThreadName,		// CAN BE NULL
						IzzyThreadInfo*			outThreadInfo);
	
	// waits for a thread to complete its task
	IzzyThreadErr	IzzyWaitThreadCompleteAndDispose(
						IzzyThreadInfo*			inThreadInfo);

	// blocks until a signal is received from some other thread
	void			IzzyDisposeThread(
						IzzyThreadInfo*			inThreadInfo);
	
	// blocks until a signal is received from some other thread
	bool			IzzyThreadBlockUntilSignal(
						IzzyThreadInfo*			inThreadInfo);
	
	// blocks until a signal is received from some other thread
	// or until the specified number of milliseconds has passed
	bool			IzzyThreadBlockUntilSignalOrTimeout(
						IzzyThreadInfo*			inThreadInfo,
						UInt32					inMilliseconds,
						bool*					outTimedOut = NULL);
	
	// sends a signal to the target thread
	void			IzzyThreadSendSignal(
						IzzyThreadInfo*			inThreadInfo);
	
	// sends a signal to the target thread
	void			IzzyThreadSetPriority(
						IzzyThreadInfo*			inThreadInfo,
						IzzyThreadPriority		inPriority);

	void			IzzyThreadSetCurrentThreadName(
						const char* inName);
	
#ifdef __cplusplus
}
#endif

#endif
