// ===========================================================================
//	MultiVideoInOutUtil			(c) 2015 Mark F. Coniglio. All rights reserved
// ===========================================================================

#ifndef _H_MultiVideoInOutUtil
#define _H_MultiVideoInOutUtil

#include "IsadoraCallbacks.h"
#include "CVideoInputMixin.h"

class CGLShader;

#define DEBUG_GRID					FALSE

#if DEBUG_GRID
	#define		grid_log_info		IzzyLogInfo_
#else
	#define		grid_log_info(...)	{ }
#endif

enum MVScaleOption {
	kScaleOption_UsePrefs = 0,
	kScaleOption_Smallest,
	kScaleOption_Largest,
	kScaleOption_MinWidth,
	kScaleOption_MaxWidth,
	kScaleOption_MinHeight,
	kScaleOption_MaxHeight
};

class CFrameFormatInfo
{
public:
	CFrameFormatInfo(
		StageIndexT		inTargetStageIndex0 = kInvalidStageIndex,
		PixelSize		inFrameSize = PixelSize()) :
			mTargetStageIndex0(inTargetStageIndex0),
			mFrameSize(inFrameSize)
	{
	}
	
	inline void		Reset()
					{
						mTargetStageIndex0 = kInvalidStageIndex;
						mFrameSize.width = 0;
						mFrameSize.height = 0;
					}
	
	StageIndexT		mTargetStageIndex0;
	PixelSize		mFrameSize;
};

#define kMultiVideoMaxShaders	4

class CMultiVideoInOut
{
public:

				CMultiVideoInOut(
					IsadoraParameters*	ip,
					ActorInfo*			inActorInfo,
					size_t				inInputCount,
					PropertyIndex		inVideoOutputPropertyIndex1,
					size_t				inFBOCount = 1);
	
				~CMultiVideoInOut();

private:
	void		CreateMultiVideoInOut(
					IsadoraParameters*	ip,
					ActorInfo*			inActorInfo,
					size_t				inInputCount,
					PropertyIndex		inVideoOutputPropertyIndex1,
					size_t				inFBOCount = 1);

public:
	
	void		Dispose();
	
	void		InvalidateVideoOutput();
	
	void		DisposeResources();
	
	void		DisposeImageResources(
					bool				inClearAllInputFrames = true);

	void		CreateFBOs(
					StageIndexT			inTargetStageIndex0,
					UInt32				inFrameWidth,
					UInt32				inFrameHeight,
					CreateFBOOptions&	inCreateFBOOptions);
	
	size_t		PrepareToCreateFBOs();
	
	void		CreateIndividualFBO(
					size_t				inIndex0,
					StageIndexT			inTargetStageIndex0,
					UInt32				inFrameWidth,
					UInt32				inFrameHeight,
					CreateFBOOptions&	inCreateFBOOptions);
	
	bool		WillDisposeBecauseOfStageMessage(
					const MsgData*		inData) const;
	
	bool		HandleAboutToDisposeStageMessage(
					const MsgData*		inData);
	
	// GETTERS + SETTERS
	// inline void					SetNeedsDraw()						{ mNeedsDraw = true; }
	// inline void					ClearNeedsDraw()					{ mNeedsDraw = false; }
	void						SetNeedsDraw();
	void						ClearNeedsDraw();
	inline bool					GetNeedsDraw() const				{ return mNeedsDraw; }

	inline void					SetHasValidOutput()					{ mHasValidOutput = true; }
	inline void					ClearHasValidOutput()				{ mHasValidOutput = false; }
	inline bool					GetHasValidOutput() const			{ return mHasValidOutput; }

	inline DataType				GetOutputVideoBufferType() const	{ return mOutputVideoBufferType; }
	
	inline size_t				GetValidVideoInputCount() const		{ return mValidVideoInputCount; }
	inline bool					HasOutputFrameBuffer() const		{ return mFBO != NULL; }
	inline IzzyFrameBuffer		GetOutputFrameBuffer() const		{ return mFBO[0]; }
	
	inline IzzyFrameBuffer		GetExtraFrameBuffer(
									size_t		inIndex0)
								{
									size_t realIndex = inIndex0 + 1;
									if (realIndex >= 1 && realIndex < mFBOCount) {
										return mFBO[realIndex];
									}
									return NULL;
								}

	inline TextureBufferPtr		GetValidInputTextureBuffer(size_t inIndex0) const
									{ return mValidInputArray[inIndex0]->GetTextureImageInput(); }

	#if COMPILE_CORE_VIDEO
	inline CoreImageBuffer&		GetOutputCoreImageBuffer()			{ return mCoreImageOutput; }
	#endif
	
	inline ImageBufferPtr		GetOutputBitmap() const				{ return mOutputBitmap; }
	
	inline StageIndexT			GetStageIndex() const				{ return mStageIndex0; }

// VIDEO INPUT
	void		SetVideoInputFromValue(
					size_t		inVideoInputIndex0,
					ValuePtr	inValue);
	
	void		PrepareInputsAndCalculateFrameSizeDirect(
					CFrameFormatInfo&	outFFI,
					bool*				outFrameSizeChanged);

	void		GetOutputResolutionForImageProcessingMode(
					IsadoraParameters*	ip,
					CFrameFormatInfo&	outFFI,
					bool*				outFrameSizeChanged,
					int					inForceFrameSizeFromInputIndex0 = -1);
					
	void		GetOutputResolutionForImageProcessingMode(
					IsadoraParameters*	ip,
					const PixelSize&	inPixelSizeOverride,
					CFrameFormatInfo&	outFFI,
					bool*				outFrameSizeChanged,
					int					inForceFrameSizeFromInputIndex0 = -1);

	void		GetOutputResolutionForImageProcessingModeWithScaleOptions(
					IsadoraParameters*	ip,
					MVScaleOption		inScaleOption,
					CFrameFormatInfo&	outFFI,
					bool*				outFrameSizeChanged);
					
// VIDEO OUTPUT
	void		SetVideoOutputTypeBasedOnProperty(
					PropertyType		inPropertyType,
					PropertyIndex		inOutputPropertyIndex1);
	
	void		SetVideoOutputType(
					PropertyDispFormat	inCurrentDisplayFormat);
	
	bool		UpdateTextureOutput(
					StageIndexT			inTargetStageIndex0,
					const PixelSize&	inOutputSize,
					bool				inInitToBlack = false,
					bool				inPowerOfTwo = true);
	
	bool		UpdateTextureOutput(
					StageIndexT			inTargetStageIndex0,
					const PixelSize&	inOutputSize,
					CreateFBOOptions&	inCreateFBOOptions);
	
	bool		UpdateIndividualFBO(
					size_t				inIndex0,
					StageIndexT			inTargetStageIndex0,
					const PixelSize&	inOutputSize,
					CreateFBOOptions&	inCreateFBOOptions);
	
	void		UpdateCoreImageOutput(
					StageIndexT			inTargetStageIndex0,
					const PixelSize&	inOutputSize);
	
	void		UpdateBitmapOutput(
					const PixelSize&	cropDstSize);
	
// BLACK TEXTURE
	bool				HasBlackTexture() const
							{ return (mBlackTextureID != 0); }
	
	bool				IsBlackTexturePremultiplied() const
							{ return false; }
	
	GLuint				GetBlackTextureID() const
							{ return mBlackTextureID; }
	
	void				CreateBlackTexture(
							/* float		inAlphaIntensity = 1.0 */);
	
	void				DisposeBlackTexture();
	
	TextureBufferPtr	GetBlackTextureBuffer()
							{ return &mBlackTextureBuffer; }
	
	void				TextureBuffer_Init(
							TextureBufferPtr		inTextureBuffer);
	
	void				TextureBuffer_Clear(
							TextureBufferPtr		inTextureBuffer);
	
	#if COMPILE_CORE_VIDEO
	void				CreateBlackCIImage(
							const PixelSize&	inPixelSize);

	void				DisposeBlackCIImage();

	CoreImageBuffer*	UpdateBlackCIImageBuffer(
							StageIndexT			inStageIndex0,
							const PixelSize&	inPixelSize);

	void				DisposeBlackCIImageBuffer();
	#endif

// 2D LOOKUP TABLE
	void		SetLoookupTable2DTarget(
					GLenum			inLoookupTable2DTarget)
						{ mLoookupTable2DTarget = inLoookupTable2DTarget; }
	
	bool		HasLookupTableData() const
					{ return (mLookupTable2DData != NULL); }
	
	bool		RequiresLookupTable2D() const
					{ return (mLookupTable2DData != NULL && mLookupTable2DSize.width > 0 && mLookupTable2DSize.height > 0); }
	
	bool		HasLookupTable2D() const
					{ return (mLookupTable2DTextureID != 0); }
	
	GLuint		GetLookupTable2DTextureID() const
					{ return mLookupTable2DTextureID; }
	
	void		SetLookupTable2DData(
					const UInt32*		inData,
					size_t				inWidth,
					size_t				inHeight,
					bool				inIsPersistent)
						{ SetLookupTable2DData((const UInt8*) inData, inWidth, inHeight, inIsPersistent, 4); }
	
	void		SetLookupTable2DData(
					const UInt8*		inData,
					size_t				inWidth,
					size_t				inHeight,
					bool				inIsPersistent,
					size_t				inPixelSize = 1);
	
	void		DisposeLookupTable2DData();

	void		UpdateLookupTable2DDataInCurrentContext(
					const UInt32*		inData,
					size_t				inWidth,
					size_t				inHeight,
					bool				inIsPersistent)
						{ UpdateLookupTable2DDataInCurrentContext((const UInt8*) inData, inWidth, inHeight, inIsPersistent, 4); }
	
	void		UpdateLookupTable2DDataInCurrentContext(
					const UInt8*		inData,
					size_t				inWidth,
					size_t				inHeight,
					bool				inIsPersistent,
					size_t				inPixelSize = 1);
	
	void		CreateLookupTable2D();
	
	void		DisposeLookupTable2D();
	
	void		DisposeLookupTableAndData();

	void		DeleteLookupTable2DInCurrentContext();

private:
	void		CreateLookupTable2DInCurrentContext();
	
	void		UpdateLookupTable2DInCurrentContext();


// GRID MESH VBO
public:

#if ENABLE_VBO_MESH_GENERATION
	bool		HasGridMeshVBO()
					{ return mGridMeshVertexVBO != 0 || mGridMeshIndexVBO != 0; }
	
	void		GenerateGridMeshVBO(
					GLenum				inMode,
					const PixelSize&	inDivisions,
					const CFloatXY&		inTexCoordMax,
					const bool			inUseIndices,
					const float*		inDisplacementTable = NULL,
					const size_t		inDisplacementTableSize = 0);

	bool		UpdateGridMeshVBO(
					GLenum				inMode,
					const PixelSize&	inDivisions,
					const CFloatXY&		inTexCoordMax,
					const bool			inUseIndices,
					const float*		inDisplacementTable = NULL,
					const size_t		inDisplacementTableSize = 0);

	void		UpdateGridMeshDisplacementMap(
					const float*		inDisplacementTable,
					const size_t		inDisplacementTableSize);
	
	void		DeleteGridMeshVBO();
	
	void		DrawGridMeshVBO(
					GLint				inDisplacementAttributeLocation = 0,
					float*				inDisplacementTable = NULL,
					float				inModulationAmount = 0.0f);
	
	void		DrawGridMeshVBOWithTranslationTable(
					const float*		inOffsetTable,
					const float			inAmount);
#endif

// UNITY QUAD
	static void	DrawUnityQuad_NoTexture();

	static void	DrawUnityQuad_Textured(
					const float			inTxCoordRight,
					const float			inTxCoordBottom);

	static void	DrawUnityQuad_Textured(
					const float			inTxCoordLeft,
					const float			inTxCoordTop,
					const float			inTxCoordRight,
					const float			inTxCoordBottom);

// SHADER CREATION/DESTRUCTION
	bool		HasShader() const
					{ return mShaders[0] != NULL; }
					
	CGLShader*	GetShader() const
					{ return mShaders[0]; }
					
	CGLShader*	GetIndexedShader(
					size_t		inIndex0) const
				{
					if (inIndex0 < kMultiVideoMaxShaders) {
						return mShaders[inIndex0];
					} else {
						return NULL;
					}
				}
					
	bool		HasIndexedShader(
					size_t		inIndex0) const
				{
					if (inIndex0 < kMultiVideoMaxShaders) {
						return mShaders[inIndex0] != NULL;
					} else {
						return false;
					}
				}
					
	void		CreateShader(
					const char*		inVertexShader,
					const char*		inFragmentShader)
				{
					CreateIndexedShader(0, inVertexShader, inFragmentShader);
				}
	
	void		DisposeAllShaders();

	void		CreateIndexedShader(
					size_t			inIndex0,
					const char*		inVertexShader,
					const char*		inFragmentShader);
	
	void		DisposeIndexedShader(
					size_t			inIndex0);

// SHADER ENABLE/DISABLE
	bool		EnableShader(
					const GLcharARB*	inParamName1 = NULL,
					int					inSize1 = 0,
					float*				inData1 = NULL)
				{
					return EnableIndexedShader(0, inParamName1, inSize1, inData1);
				}
	
	void		DisableShader()
				{
					DisableIndexedShader(0);
				}

// SHADER ENABLE/DISABLE
	bool		EnableIndexedShader(
					size_t				inIndex0,
					const GLcharARB*	inParamName1 = NULL,
					int					inSize1 = 0,
					float*				inData1 = NULL);
	
	void		DisableIndexedShader(
					size_t				inIndex0);

// VIDEO OUTPUT
	void		SetVideoOutput(
					const ValuePtr		inValue);
	
private:
	void		DisposeFBOs();
	
public:

	IsadoraParameters*		mIP;
	ActorInfo*				mActorInfo;
	
	// IMAGE PROCESSING
	size_t					mVideoInputCount;
	CVideoInputMixin*		mVideoInputArray;

	// VALIDATED VIDEO INPUTS
	size_t					mValidVideoInputCount;
	CVideoInputMixinPtr*	mValidInputArray;
	size_t*					mValidVideoInputIndexArray;
	
	// OUTPUT BUFFERS
	
	// TEXTURE
	size_t					mFBOCount;
	IzzyFrameBufferPtr		mFBO;				// an array of FBO ptrs
	StageIndexT				mStageIndex0;
	
	// CORE IMAGE
	#if COMPILE_CORE_VIDEO
	CoreImageBuffer			mCoreImageOutput;
	#endif

	// BITMAP
	ImageBufferPtr			mOutputBitmap;

	// SHADER
	CGLShader*				mShaders[kMultiVideoMaxShaders];
	
	// 2D LOOKUP TABLE
	GLenum					mLoookupTable2DTarget;
	const UInt8*			mLookupTable2DData;
	PixelSize				mLookupTable2DSize;
	size_t					mLookupTable2DPixelSize;
	GLuint					mLookupTable2DTextureID;
	bool					mOwnsLookupTable2D;

	// BLACK TEXTURE
	GLuint					mBlackTextureID;
	TextureBuffer			mBlackTextureBuffer;
	
	// BLACK CI_IMAGE
	#if COMPILE_CORE_VIDEO
	CI_Image				mBlackCIImage;
	CoreImageBuffer*		mBlackCIImageBuffer;
	PixelSize				mBlackCIImageBufferSize;
	#endif
	
	// GRID MESH
#if ENABLE_VBO_MESH_GENERATION
	GLenum					mGridMeshDrawMode;
	GLuint					mGridMeshVertexVBO;
	GLuint					mGridMeshIndexVBO;
	GLuint					mGridDisplaceTableVBO;
	CFloatXY				mGridMeshTexCoordMax;
	PixelSize				mGridMeshDivisionSize;
	int						mGridMeshIndexCount;
	void*					mGridMeshIndexData;
	void*					mGridMeshVertexData;
#endif
	
	// OUTPUTS
	PropertyIndex			mVideoOutputPropertyIndex1;
	DataType				mOutputVideoBufferType;

	bool					mHasValidOutput;

private:
	bool					mNeedsDraw;

public:
	PixelSize				mLastDstSize;
};

inline bool
TextureOKToDraw(
	IsadoraParameters* ip,
	TextureBufferPtr tb,
	const CFrameFormatInfo&	inFFI)
{
	return (ip->mSharedGLContexts || tb->mTargetStageIndex0 == inFFI.mTargetStageIndex0);
}

inline bool
ImageOKToDraw(
	IsadoraParameters* ip,
	CoreImageBufferPtr ci,
	const CFrameFormatInfo&	inFFI)
{
	return (ip->mSharedGLContexts || ci->mTargetStageIndex0 == inFFI.mTargetStageIndex0);
}

#endif
