// ===========================================================================
// ThreadUtils					(c) 2006 Mark F. Coniglio.  all rights reserved.
// ===========================================================================
// Cross platform threading for Mac OS X and Windows

#ifndef _H_ThreadUtils
#define _H_ThreadUtils

#ifdef __cplusplus
extern "C" {
#endif

#if TARGET_OS_MAC

	#include <pthread.h>
	#include <mach/mach_init.h>
	#include <mach/thread_policy.h>

	#define	msSleep(a)				usleep((a) * 1000);

#elif TARGET_OS_WIN

	#define	msSleep(a)				Sleep(a)
	
#else

	#error "Platform Must Be Mac or Win!"

#endif

	// returns true if there are multiple processors available on this machine
	// bool				MultipleProcessorsAvailable(); // 3.0.8b9 -- removed
	
	// returns the total number of processors on this machine
	// UInt32				GetProcessorCount(); // 3.0.8b9 -- removed
	
//	void				IzzyThreadSetCurrentThreadName(
//							const char* inName);
	
	// returns true if called from the main thread
	inline bool			IsRunningInForeground()
						{
							#if TARGET_OS_MAC
								return pthread_main_np();
							#elif TARGET_OS_WIN
								extern DWORD gWindowsMainThreadID;
								return GetCurrentThreadId() == gWindowsMainThreadID;
							#endif
						}
	
	// returns true if called from a background thread
	inline bool			IsRunningInBackground()
						{
							#if TARGET_OS_MAC
								return !pthread_main_np();
							#elif TARGET_OS_WIN
								extern DWORD gWindowsMainThreadID;
								return GetCurrentThreadId() != gWindowsMainThreadID;
							#endif
						}
	
	inline const char*	ThreadText()
						{
							extern const char* sForegroundThreadText;
							extern const char* sBackgroundThreadText;
							return (IsRunningInForeground() ? sForegroundThreadText : sBackgroundThreadText);
						}
						
#ifdef __cplusplus
}
#endif

#endif
